package ru.scancode.GodexPrintTest;

import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import android.util.SparseArray;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.TextView;

import utils.dialogDirPicker;

//Класс для реализации печати из CMD файла

public class pagePrintCmdFile extends AbstractPage implements variablePromptDialog.IEditorListener{
    private TextView editPathCmd;
    private TextView editPathSrc;
    private final int REQUEST_CMD=0;
    private final int REQUEST_SRC=1;
    private String pathCmd ="";
    private String pathSrc ="";

    private View.OnClickListener clickListener= new View.OnClickListener() {
        @Override
        public void onClick(View v) {
            Intent intent = new Intent(getContext(), utils.dialogDirPicker.class);
            intent.addCategory(Intent.CATEGORY_DEFAULT);

            switch(v.getId()){
                case R.id.btnGetPath:
                    intent.putExtra(dialogDirPicker.EXTRA_FILTER, ".cmd");
                    intent.putExtra(dialogDirPicker.EXTRA_BEGIN_DIR, editPathCmd.getText().toString());
                    startActivityForResult(intent, REQUEST_CMD);
                break;
                case R.id.btnPickSource:
                    intent.putExtra(dialogDirPicker.EXTRA_FILTER, ".txt;.xls;.xml;.db");
                    intent.putExtra(dialogDirPicker.EXTRA_BEGIN_DIR, editPathSrc.getText().toString());
                    startActivityForResult(intent, REQUEST_SRC);
                    break;
            }
        }
    };

    @Override
    public void onActivityResult(int requestCode, int resultCode, Intent data) {
        if(resultCode== Activity.RESULT_OK) {
            switch (requestCode) {
                case REQUEST_CMD:
                    pathCmd = data.getStringExtra(utils.dialogDirPicker.EXTRA_RESULT);
                    break;
                case REQUEST_SRC:
                    pathSrc = data.getStringExtra(utils.dialogDirPicker.EXTRA_RESULT);
                    break;
            }
            editPathCmd.setText(pathCmd);
            editPathSrc.setText(pathSrc);
        }
    }

    @Override
    public View onCreateView(LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        View result=null;
        if(inflater!=null) {
            result = inflater.inflate(R.layout.print_cmd_file_page, container, false);

            editPathCmd = result.findViewById(R.id.editFilePath);
            editPathSrc = result.findViewById(R.id.editSrcPath);
            Button btnPickFile = result.findViewById(R.id.btnGetPath);
            Button btnGetSource = result.findViewById(R.id.btnPickSource);

            btnPickFile.setOnClickListener(clickListener);
            btnGetSource.setOnClickListener(clickListener);
            btnPickFile.requestFocus();
        }
        return result;
    }

    @Override
    public SparseArray<variable> getVariables() {
        return variables;
    }

    private data_packer InitPrintData(String template){
        //Начальная инициализация пакета с данными для посылки в service
        data_packer packer = new data_packer();
        try {
            packer.setCommand(data_packer.commands.CMD_PRINT);
            packer.addParam(data_packer.parameters.PARAM_QUANTITY, data_packer.values.VAL_COUNT, "", m_quantity);
            param = packer.addParam(data_packer.parameters.PARAM_CMD_FILE, template);
        }
        catch(Exception e){
            utils.logger.get().write(e.toString() + " " + e.getMessage());
        }
        return packer;
    }

    @Override
    public void PromptVariables(SparseArray<variable> vars) {
        pathCmd=editPathCmd.getText().toString();
        packer =InitPrintData(pathCmd);

        Bundle args= new Bundle();
        args.putSparseParcelableArray(EXTRA_VARIABLE, vars);

        dlg = new variablePromptDialog();
        dlg.setArguments(args);
        dlg.setTargetFragment(this, REQUEST_ANSWER_CODE);
        dlg.show(m_parentActivity.getSupportFragmentManager(), "");
        bPromptDialogIsShown=true;
    }

    //Возвращает строку XML в MainActivity при нажатии кнопки ПЕЧАТЬ
    @Override
    public String getPrintData() {
        String result="";
        try {
            pathCmd=editPathCmd.getText().toString(); //Путь к cmd-файлу
            pathSrc=editPathSrc.getText().toString(); //Путь к файлу-источнику данных

            packer =InitPrintData(pathCmd);

            if(pathSrc!=null && !pathSrc.isEmpty()) {
                param.addValue(data_packer.values.VAL_PATH, null, pathSrc); //путь к файлу-источнику данных
            }
            result= packer.toString();

        } catch (Exception e) {
            utils.logger.get().write(e.getMessage());
        }
        return result;
    }

    @Override
    public void OnDissmissDalog() {
        bPromptDialogIsShown=false;
    }

    @Override
    public void OnDialogResult(@NonNull SparseArray<variable> result) {

        param.clear();
        pathCmd=editPathCmd.getText().toString();
        param.addValue(data_packer.values.VAL_PATH, null, pathCmd);

//        Iterator<String> it=result.keySet().iterator();
//        int i=0;
//        while(it.hasNext()){
//            String key=it.next();
//            String value=result.get(key).value;
//            String varName="V" + i;
//            param.addValue(data_packer.values.VAL_VARIABLE, varName, value);
//            i++;
//        }

        for(int i=0; i<result.size(); i++){
            variable v=result.get(i);
            String varName="V" + i;
            param.addValue(data_packer.values.VAL_VARIABLE, varName, v.value);
        }
        printAdapter.getInstance(m_parentActivity).sendServiceCmd(packer.toString());
    }

    private void dismissDlg(){
        if(dlg!=null){
            dlg.dismissAllowingStateLoss();
        }
    }

    @Override
    public void onPause() {
        super.onPause();
        dismissDlg();
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        dismissDlg();
    }
}
